#ifndef lint
static const char yysccsid[] = "@(#)yaccpar	1.9 (Berkeley) 02/21/93";
#endif

#include <stdlib.h>
#include <string.h>

#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYPATCH 20070509

#define YYEMPTY (-1)
#define yyclearin    (yychar = YYEMPTY)
#define yyerrok      (yyerrflag = 0)
#define YYRECOVERING (yyerrflag != 0)

extern int yyparse(void);

static int yygrowstack(void);
#define YYPREFIX "yy"
#line 22 "cfgfile_parser.y"
#include "csync2.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fnmatch.h>
#include <ctype.h>

struct csync_group  *csync_group  = 0;
struct csync_prefix *csync_prefix = 0;
struct csync_nossl  *csync_nossl  = 0;

int csync_ignore_uid = 0;
int csync_ignore_gid = 0;
int csync_ignore_mod = 0;
unsigned csync_lock_timeout = 12;
char *csync_tempdir = NULL;

#ifdef __CYGWIN__
int csync_lowercyg_disable = 0;
int csync_lowercyg_used = 0;
#endif

extern void yyerror(char* text);
extern int yylex();
extern int yylineno;

void yyerror(char *text)
{
	csync_fatal("Near line %d: %s\n", yylineno, text);
}

static void new_group(char *name)
{
	int static autonum = 1;
	int rc = 0;
	struct csync_group *t =
		calloc(sizeof(struct csync_group), 1);

	if (name == 0)
		rc = asprintf(&name, "group_%d", autonum++);
	if (!t || rc == -1)
		csync_fatal("Out of memory while parsing config file.\n");

	t->next = csync_group;
	t->auto_method = -1;
	t->gname = name;
	t->backup_generations = 3;
	csync_group = t;
}

static void add_host(char *hostname, char *peername, int slave)
{
	int i;
	for (i=0; hostname[i]; i++)
		hostname[i] = tolower(hostname[i]);
	for (i=0; peername[i]; i++)
		peername[i] = tolower(peername[i]);
	if ( strcmp(hostname, myhostname) == 0 ) {
		csync_group->local_slave = slave;
		csync_group->myname = peername;
		free(hostname);
	} else {
		struct csync_group_host *t =
			calloc(sizeof(struct csync_group_host), 1);
		t->hostname = peername;
		t->on_left_side = !csync_group->myname;
		t->slave = slave;
		t->next = csync_group->host;
		csync_group->host = t;
		free(hostname);
	}
}

static void add_patt(int patterntype, char *pattern)
{
	struct csync_group_pattern *t =
		calloc(sizeof(struct csync_group_pattern), 1);
	int i;

#if __CYGWIN__
	if (isalpha(pattern[0]) && pattern[1] == ':' &&
	    (pattern[2] == '/' || pattern[2] == '\\')) {
		char *new_pattern, *p;
		asprintf(&new_pattern, "/cygdrive/%c/%s",
			tolower(pattern[0]), pattern+3);
		for (p = new_pattern; *p; p++)
			if (*p == '\\') *p = '/';
		free(pattern);
		pattern = new_pattern;
	}
#endif

	/* strip trailing slashes from pattern */
	for (i=strlen(pattern)-1; i>0; i--)
		if (pattern[i] == '/')
			pattern[i] = 0;
		else
			break;

	/* if you use ** at least once anywhere in the pattern,
	 * _all_ stars in the pattern, even single ones,
	 * will match slashes. */
	t->star_matches_slashes = !!strstr(pattern, "**");
	t->isinclude = patterntype >= 1;
	t->iscompare = patterntype >= 2;
	t->pattern = pattern;
	t->next = csync_group->pattern;
	csync_group->pattern = t;
}

static void set_key(char *keyfilename)
{
	FILE *keyfile;
	char *fname = keyfilename;
	char line[1024];
	int i;

	if ( csync_group->key )
		csync_fatal("Config error: a group might only have one key.\n");

	if (fname && fname[0] && fname[0] != '/')
		ASPRINTF(&fname, "%s/%s", systemdir, keyfilename);

	if ( (keyfile = fopen(fname, "r")) == 0 ||
	     fgets(line, 1024, keyfile) == 0 )
		csync_fatal("Config error: Can't read keyfile %s.\n", fname);

	for (i=0; line[i]; i++) {
		if (line[i] == '\n') { line[i]=0; break; }
		if ( !(line[i] >= 'A' && line[i] <= 'Z') &&
		     !(line[i] >= 'a' && line[i] <= 'z') &&
		     !(line[i] >= '0' && line[i] <= '9') &&
		     line[i] != '.' && line[i] != '_' )
			csync_fatal("Unallowed character '%c' in key file %s.\n",
					line[i], fname);
	}

	if ( strlen(line) < 32 )
		csync_fatal("Config error: Key in file %s is too short.\n", fname);

	csync_group->key = strdup(line);
	if (fname != keyfilename)
		free(fname);
	free(keyfilename);
	fclose(keyfile);
}

static void set_auto(char *auto_method)
{
	int method_id = -1;

	if (csync_group->auto_method >= 0)
		csync_fatal("Config error: a group might only have one auto-setting.\n");

	if (!strcmp(auto_method, "none"))
		method_id = CSYNC_AUTO_METHOD_NONE;

	if (!strcmp(auto_method, "first"))
		method_id = CSYNC_AUTO_METHOD_FIRST;

	if (!strcmp(auto_method, "younger"))
		method_id = CSYNC_AUTO_METHOD_YOUNGER;

	if (!strcmp(auto_method, "older"))
		method_id = CSYNC_AUTO_METHOD_OLDER;

	if (!strcmp(auto_method, "bigger"))
		method_id = CSYNC_AUTO_METHOD_BIGGER;

	if (!strcmp(auto_method, "smaller"))
		method_id = CSYNC_AUTO_METHOD_SMALLER;

	if (!strcmp(auto_method, "left"))
		method_id = CSYNC_AUTO_METHOD_LEFT;

	if (!strcmp(auto_method, "right"))
		method_id = CSYNC_AUTO_METHOD_RIGHT;

	if (method_id < 0)
		csync_fatal("Config error: Unknown auto-setting '%s' (use "
			"'none', 'younger', 'older', 'bigger', 'smaller', "
			"'left' or 'right').\n", auto_method);

	csync_group->auto_method = method_id;
	free(auto_method);
}

static void set_bak_dir(char *dir)
{
	csync_group->backup_directory = dir;
}

static void set_bak_gen(char *gen)
{
	csync_group->backup_generations = atoi(gen);
	free(gen);
}

static void check_group()
{
	if ( ! csync_group->key )
		csync_fatal("Config error: every group must have a key.\n");

	if ( csync_group->auto_method < 0 )
		csync_group->auto_method = CSYNC_AUTO_METHOD_NONE;

	/* re-order hosts and pattern */
	{
		struct csync_group_host *t = csync_group->host;
		csync_group->host = 0;
		while ( t ) {
			struct csync_group_host *next = t->next;
			t->next = csync_group->host;
			csync_group->host = t;
			t = next;
		}
	}
	{
		struct csync_group_pattern *t = csync_group->pattern;
		csync_group->pattern = 0;
		while ( t ) {
			struct csync_group_pattern *next = t->next;
			t->next = csync_group->pattern;
			csync_group->pattern = t;
			t = next;
		}
	}

	if (active_peerlist) {
		struct csync_group_host *h;
		int i=0, thisplen;

		while (active_peerlist[i]) {
			thisplen = strcspn(active_peerlist + i, ",");

			for (h=csync_group->host; h; h=h->next)
				if (strlen(h->hostname) == thisplen && !strncmp(active_peerlist + i, h->hostname, thisplen))
					goto foundactivepeers;

			i += thisplen;
			while (active_peerlist[i] == ',') i++;
		}
	} else
foundactivepeers:
		csync_group->hasactivepeers = 1;

	if (active_grouplist && csync_group->myname)
	{
		int i=0, gnamelen = strlen(csync_group->gname);

		while (active_grouplist[i])
		{
			if ( !strncmp(active_grouplist+i, csync_group->gname, gnamelen) &&
			     (active_grouplist[i+gnamelen] == ',' || !active_grouplist[i+gnamelen]) )
				goto found_asactive;
			while (active_grouplist[i])
				if (active_grouplist[i++]==',') break;
		}

		csync_group->myname = 0;
found_asactive:	;
	}
}

static void new_action()
{
	struct csync_group_action *t =
		calloc(sizeof(struct csync_group_action), 1);
	t->next = csync_group->action;
	t->logfile = "/dev/null";
	csync_group->action = t;
}

static void add_action_pattern(const char *pattern)
{
	struct csync_group_action_pattern *t =
		calloc(sizeof(struct csync_group_action_pattern), 1);
	t->star_matches_slashes = !!strstr(pattern, "**");
	t->pattern = pattern;
	t->next = csync_group->action->pattern;
	csync_group->action->pattern = t;
}

static void add_action_exec(const char *command)
{
	struct csync_group_action_command *t =
		calloc(sizeof(struct csync_group_action_command), 1);
	t->command = command;
	t->next = csync_group->action->command;
	csync_group->action->command = t;
}

static void set_action_logfile(const char *logfile)
{
	csync_group->action->logfile = logfile;
}

static void set_action_dolocal()
{
	csync_group->action->do_local = 1;
}

static void set_action_dolocal_only()
{
	csync_group->action->do_local = 1;
	csync_group->action->do_local_only = 1;
}

static void set_lock_timeout(const char *timeout)
{
	csync_lock_timeout = atoi(timeout);
}

static void set_tempdir(const char *tempdir)
{
	csync_tempdir = strdup(tempdir);
}

static void set_database(const char *filename)
{
	if (!csync_database)
		csync_database = strdup(filename);
}

static void new_prefix(const char *pname)
{
	struct csync_prefix *p =
		calloc(sizeof(struct csync_prefix), 1);
	p->name = pname;
	p->next = csync_prefix;
	csync_prefix = p;
}

static void new_prefix_entry(char *pattern, char *path)
{
	int i;

	if (path[0] != '/')
		csync_fatal("Config error: Prefix '%s' is not an absolute path.\n", path);

	if (!csync_prefix->path && !fnmatch(pattern, myhostname, 0)) {
#if __CYGWIN__
		if (isalpha(path[0]) && path[1] == ':' &&
		    (path[2] == '/' || path[2] == '\\')) {
			char *new_path, *p;
			asprintf(&new_path, "/cygdrive/%c/%s",
				tolower(path[0]), path+3);
			for (p = new_path; *p; p++)
				if (*p == '\\') *p = '/';
			free(path);
			path = new_path;
		}
#endif
		for (i=strlen(path)-1; i>0; i--)
			if (path[i] == '/')
				path[i] = 0;
			else
				break;

		csync_debug(2, "Prefix '%s' is set to '%s'.\n", csync_prefix->name, path);
		csync_prefix->path = path;
	} else
		free(path);
	free(pattern);
}

static void new_nossl(const char *from, const char *to)
{
	struct csync_nossl *t =
		calloc(sizeof(struct csync_nossl), 1);
	t->pattern_from = from;
	t->pattern_to = to;
	t->next = csync_nossl;
	csync_nossl = t;
}

static void new_ignore(char *propname)
{
	if ( !strcmp(propname, "uid") )
		csync_ignore_uid = 1;
	else
	if ( !strcmp(propname, "gid") )
		csync_ignore_gid = 1;
	else
	if ( !strcmp(propname, "mod") )
		csync_ignore_mod = 1;
	else
		csync_fatal("Config error: Unknown 'ignore' porperty: '%s'.\n", propname);

	free(propname);
}

static void disable_cygwin_lowercase_hack()
{
#ifdef __CYGWIN__
	if (csync_lowercyg_used)
		csync_fatal("Config error: 'nocygwinlowercase' must be at the top of the config file.\n");
	csync_lowercyg_disable = 1;
#else
	csync_fatal("Config error: Found 'nocygwinlowercase' but this is not a cygwin csync2.\n");
#endif
}

#line 429 "cfgfile_parser.y"
typedef union {
	char *txt;
} YYSTYPE;
#line 431 "cfgfile_parser.c"
#define TK_BLOCK_BEGIN 257
#define TK_BLOCK_END 258
#define TK_STEND 259
#define TK_AT 260
#define TK_AUTO 261
#define TK_NOSSL 262
#define TK_IGNORE 263
#define TK_GROUP 264
#define TK_HOST 265
#define TK_EXCL 266
#define TK_INCL 267
#define TK_COMP 268
#define TK_KEY 269
#define TK_DATABASE 270
#define TK_ACTION 271
#define TK_PATTERN 272
#define TK_EXEC 273
#define TK_DOLOCAL 274
#define TK_LOGFILE 275
#define TK_NOCYGLOWER 276
#define TK_PREFIX 277
#define TK_ON 278
#define TK_COLON 279
#define TK_POPEN 280
#define TK_PCLOSE 281
#define TK_BAK_DIR 282
#define TK_BAK_GEN 283
#define TK_DOLOCALONLY 284
#define TK_TEMPDIR 285
#define TK_LOCK_TIMEOUT 286
#define TK_STRING 287
#define YYERRCODE 256
short yylhs[] = {                                        -1,
    0,    0,    1,    4,    1,    1,    1,    1,    1,    1,
    1,    6,    6,    5,    5,    2,    2,    3,    7,    7,
    7,    8,    8,    8,    8,    8,    8,    8,    8,   10,
   10,   10,   10,   14,   14,   14,   11,   11,   12,   12,
   13,   13,   15,    9,   16,   16,   17,   17,   17,   17,
   17,   18,   18,   19,   19,
};
short yylen[] = {                                         2,
    0,    2,    2,    0,    6,    4,    3,    3,    3,    2,
    3,    0,    2,    0,    6,    1,    2,    3,    0,    3,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    0,
    2,    4,    5,    0,    2,    4,    0,    2,    0,    2,
    0,    2,    0,    5,    0,    3,    2,    2,    2,    1,
    1,    0,    2,    0,    2,
};
short yydefred[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   17,    0,   10,    4,    0,    0,
    2,    0,    3,    0,   13,    9,    7,    0,    8,   11,
    0,   30,   37,   39,    0,    0,   43,    0,    0,    0,
    0,    0,    6,   14,   27,    0,    0,    0,    0,   25,
   26,    0,   28,   29,   18,    0,   21,    0,   34,    0,
   38,   40,    0,    0,   20,    5,    0,    0,    0,   52,
   54,   50,    0,   51,    0,    0,    0,   30,    0,   32,
    0,    0,   49,   44,    0,    0,    0,    0,   53,   55,
   46,    0,   36,   15,
};
short yydgoto[] = {                                       9,
   10,   11,   23,   28,   58,   14,   40,   41,   42,   46,
   47,   48,   50,   68,   52,   75,   76,   81,   82,
};
short yysindex[] = {                                   -254,
 -281, -280, -269, -268, -232, -252, -243, -242,    0, -254,
 -223, -241, -280, -212,    0, -211,    0,    0, -210, -209,
    0, -228,    0, -208,    0,    0,    0, -205,    0,    0,
 -234,    0,    0,    0,    0, -231,    0, -230, -229, -199,
 -198, -228,    0,    0,    0, -276, -227, -225, -224,    0,
    0, -193,    0,    0,    0, -228,    0, -253,    0, -195,
    0,    0,    0, -260,    0,    0, -221, -261, -220,    0,
    0,    0, -219,    0, -189, -188, -207,    0, -190,    0,
 -214, -213,    0,    0, -260, -206, -276, -204,    0,    0,
    0, -184,    0,    0,
};
short yyrindex[] = {                                     76,
    0, -182, -179,    0,    0,    0,    0,    0,    0,   76,
    0,    0, -182,    0,    0,    0,    0,    0,    0,    0,
    0, -178,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, -258,    0,    0,    0,    0,    0,
    0, -178,    0,    0,    0, -180, -177, -175,    0,    0,
    0,    0,    0,    0,    0, -178,    0,    0,    0, -259,
    0,    0, -257, -173,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, -245,    0,
 -172, -171,    0,    0, -173,    0, -170,    0,    0,    0,
    0,    0,    0,    0,
};
short yygindex[] = {                                     80,
    0,    0,    0,    0,    0,   73,  -39,    0,    0,   13,
    0,   57,    0,    0,    0,    8,    0,    0,    0,
};
#define YYTABLESIZE 93
short yytable[] = {                                      31,
   41,   42,   57,   59,   66,   12,   13,    1,    2,    3,
   60,   70,   71,   72,   73,    4,   65,   15,   16,   78,
   31,    5,    6,   74,   67,   79,   17,   31,   39,   40,
    7,    8,   31,   22,   18,   35,   32,   33,   34,   35,
   36,   35,   37,   19,   20,   24,   26,   27,   29,   30,
   43,   44,   45,   38,   39,   51,   53,   54,   55,   61,
   56,   62,   63,   64,   69,   77,   80,   83,   84,   88,
   85,   86,   89,   90,   94,    1,   12,   16,   22,   19,
   92,   23,   93,   24,   45,   25,   47,   48,   33,   21,
   87,   49,   91,
};
short yycheck[] = {                                     259,
  259,  259,   42,  280,  258,  287,  287,  262,  263,  264,
  287,  272,  273,  274,  275,  270,   56,  287,  287,  281,
  280,  276,  277,  284,  278,  287,  259,  287,  287,  287,
  285,  286,  261,  257,  287,  281,  265,  266,  267,  268,
  269,  287,  271,  287,  287,  287,  259,  259,  259,  259,
  259,  257,  287,  282,  283,  287,  287,  287,  258,  287,
  259,  287,  287,  257,  260,  287,  287,  287,  258,  260,
  259,  279,  287,  287,  259,    0,  259,  257,  259,  258,
  287,  259,  287,  259,  258,   13,  259,  259,  259,   10,
   78,   35,   85,
};
#define YYFINAL 9
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 287
#if YYDEBUG
char *yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"TK_BLOCK_BEGIN","TK_BLOCK_END",
"TK_STEND","TK_AT","TK_AUTO","TK_NOSSL","TK_IGNORE","TK_GROUP","TK_HOST",
"TK_EXCL","TK_INCL","TK_COMP","TK_KEY","TK_DATABASE","TK_ACTION","TK_PATTERN",
"TK_EXEC","TK_DOLOCAL","TK_LOGFILE","TK_NOCYGLOWER","TK_PREFIX","TK_ON",
"TK_COLON","TK_POPEN","TK_PCLOSE","TK_BAK_DIR","TK_BAK_GEN","TK_DOLOCALONLY",
"TK_TEMPDIR","TK_LOCK_TIMEOUT","TK_STRING",
};
char *yyrule[] = {
"$accept : config",
"config :",
"config : block config",
"block : block_header block_body",
"$$1 :",
"block : TK_PREFIX TK_STRING $$1 TK_BLOCK_BEGIN prefix_list TK_BLOCK_END",
"block : TK_NOSSL TK_STRING TK_STRING TK_STEND",
"block : TK_DATABASE TK_STRING TK_STEND",
"block : TK_TEMPDIR TK_STRING TK_STEND",
"block : TK_IGNORE ignore_list TK_STEND",
"block : TK_NOCYGLOWER TK_STEND",
"block : TK_LOCK_TIMEOUT TK_STRING TK_STEND",
"ignore_list :",
"ignore_list : TK_STRING ignore_list",
"prefix_list :",
"prefix_list : prefix_list TK_ON TK_STRING TK_COLON TK_STRING TK_STEND",
"block_header : TK_GROUP",
"block_header : TK_GROUP TK_STRING",
"block_body : TK_BLOCK_BEGIN stmts TK_BLOCK_END",
"stmts :",
"stmts : stmt TK_STEND stmts",
"stmts : action stmts",
"stmt : TK_HOST host_list",
"stmt : TK_EXCL excl_list",
"stmt : TK_INCL incl_list",
"stmt : TK_COMP comp_list",
"stmt : TK_KEY TK_STRING",
"stmt : TK_AUTO TK_STRING",
"stmt : TK_BAK_DIR TK_STRING",
"stmt : TK_BAK_GEN TK_STRING",
"host_list :",
"host_list : host_list TK_STRING",
"host_list : host_list TK_STRING TK_AT TK_STRING",
"host_list : host_list TK_POPEN host_list_slaves TK_PCLOSE host_list",
"host_list_slaves :",
"host_list_slaves : host_list_slaves TK_STRING",
"host_list_slaves : host_list_slaves TK_STRING TK_AT TK_STRING",
"excl_list :",
"excl_list : excl_list TK_STRING",
"incl_list :",
"incl_list : incl_list TK_STRING",
"comp_list :",
"comp_list : incl_list TK_STRING",
"$$2 :",
"action : TK_ACTION $$2 TK_BLOCK_BEGIN action_stmts TK_BLOCK_END",
"action_stmts :",
"action_stmts : action_stmt TK_STEND action_stmts",
"action_stmt : TK_PATTERN action_pattern_list",
"action_stmt : TK_EXEC action_exec_list",
"action_stmt : TK_LOGFILE TK_STRING",
"action_stmt : TK_DOLOCAL",
"action_stmt : TK_DOLOCALONLY",
"action_pattern_list :",
"action_pattern_list : action_pattern_list TK_STRING",
"action_exec_list :",
"action_exec_list : action_exec_list TK_STRING",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif

/* define the initial stack-sizes */
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH  YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH  10000
#endif
#endif

#define YYINITSTACKSIZE 500

int      yydebug;
int      yynerrs;
int      yyerrflag;
int      yychar;
short   *yyssp;
YYSTYPE *yyvsp;
YYSTYPE  yyval;
YYSTYPE  yylval;

/* variables for the parser stack */
static short   *yyss;
static short   *yysslim;
static YYSTYPE *yyvs;
static int      yystacksize;
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack(void)
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;

    i = yyssp - yyss;
    newss = (yyss != 0)
          ? (short *)realloc(yyss, newsize * sizeof(*newss))
          : (short *)malloc(newsize * sizeof(*newss));
    if (newss == 0)
        return -1;

    yyss  = newss;
    yyssp = newss + i;
    newvs = (yyvs != 0)
          ? (YYSTYPE *)realloc(yyvs, newsize * sizeof(*newvs))
          : (YYSTYPE *)malloc(newsize * sizeof(*newvs));
    if (newvs == 0)
        return -1;

    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab
int
yyparse(void)
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")) != 0)
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = YYEMPTY;

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate]) != 0) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = YYEMPTY;
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;

    yyerror("syntax error");

#ifdef lint
    goto yyerrlab;
#endif

yyerrlab:
    ++yynerrs;

yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = YYEMPTY;
        goto yyloop;
    }

yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    if (yym)
        yyval = yyvsp[1-yym];
    else
        memset(&yyval, 0, sizeof yyval);
    switch (yyn)
    {
case 4:
#line 452 "cfgfile_parser.y"
{ new_prefix(yyvsp[0].txt); }
break;
case 5:
#line 454 "cfgfile_parser.y"
{ }
break;
case 6:
#line 456 "cfgfile_parser.y"
{ new_nossl(yyvsp[-2].txt, yyvsp[-1].txt); }
break;
case 7:
#line 458 "cfgfile_parser.y"
{ set_database(yyvsp[-1].txt); }
break;
case 8:
#line 460 "cfgfile_parser.y"
{ set_tempdir(yyvsp[-1].txt); }
break;
case 10:
#line 463 "cfgfile_parser.y"
{ disable_cygwin_lowercase_hack(); }
break;
case 11:
#line 465 "cfgfile_parser.y"
{ set_lock_timeout(yyvsp[-1].txt); }
break;
case 13:
#line 471 "cfgfile_parser.y"
{ new_ignore(yyvsp[-1].txt); }
break;
case 15:
#line 477 "cfgfile_parser.y"
{ new_prefix_entry(yyvsp[-3].txt, on_cygwin_lowercase(yyvsp[-1].txt)); }
break;
case 16:
#line 482 "cfgfile_parser.y"
{ new_group(0);  }
break;
case 17:
#line 484 "cfgfile_parser.y"
{ new_group(yyvsp[0].txt); }
break;
case 18:
#line 489 "cfgfile_parser.y"
{ check_group(); }
break;
case 26:
#line 504 "cfgfile_parser.y"
{ set_key(yyvsp[0].txt); }
break;
case 27:
#line 506 "cfgfile_parser.y"
{ set_auto(yyvsp[0].txt); }
break;
case 28:
#line 508 "cfgfile_parser.y"
{ set_bak_dir(yyvsp[0].txt); }
break;
case 29:
#line 510 "cfgfile_parser.y"
{ set_bak_gen(yyvsp[0].txt); }
break;
case 31:
#line 516 "cfgfile_parser.y"
{ add_host(yyvsp[0].txt, strdup(yyvsp[0].txt), 0); }
break;
case 32:
#line 518 "cfgfile_parser.y"
{ add_host(yyvsp[-2].txt, yyvsp[0].txt, 0); }
break;
case 35:
#line 525 "cfgfile_parser.y"
{ add_host(yyvsp[0].txt, strdup(yyvsp[0].txt), 1); }
break;
case 36:
#line 527 "cfgfile_parser.y"
{ add_host(yyvsp[-2].txt, yyvsp[0].txt, 1); }
break;
case 38:
#line 533 "cfgfile_parser.y"
{ add_patt(0, on_cygwin_lowercase(yyvsp[0].txt)); }
break;
case 40:
#line 539 "cfgfile_parser.y"
{ add_patt(1, on_cygwin_lowercase(yyvsp[0].txt)); }
break;
case 42:
#line 545 "cfgfile_parser.y"
{ add_patt(2, on_cygwin_lowercase(yyvsp[0].txt)); }
break;
case 43:
#line 550 "cfgfile_parser.y"
{ new_action(); }
break;
case 49:
#line 564 "cfgfile_parser.y"
{ set_action_logfile(yyvsp[0].txt); }
break;
case 50:
#line 566 "cfgfile_parser.y"
{ set_action_dolocal(); }
break;
case 51:
#line 568 "cfgfile_parser.y"
{ set_action_dolocal_only(); }
break;
case 53:
#line 574 "cfgfile_parser.y"
{ add_action_pattern(on_cygwin_lowercase(yyvsp[0].txt)); }
break;
case 55:
#line 580 "cfgfile_parser.y"
{ add_action_exec(yyvsp[0].txt); }
break;
#line 957 "cfgfile_parser.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;

yyoverflow:
    yyerror("yacc stack overflow");

yyabort:
    return (1);

yyaccept:
    return (0);
}
